@@ -10,7 +10,7 @@
 # copyright at the end of the file in the pod section
 
 package Config::Model::TkUI;
-$Config::Model::TkUI::VERSION = '1.347';
+$Config::Model::TkUI::VERSION = '1.348';
 use 5.10.1;
 use strict;
 use warnings;
@@ -434,6 +434,7 @@ sub check_end {
 
 sub save {
     my $cw = shift;
+    my $cb = shift || sub {};
 
     my $dir       = $cw->{save_dir};
     my $trace_dir = defined $dir ? $dir : 'default';
@@ -460,6 +461,7 @@ sub save {
         else {
             $cw->show_message("Save done ...");
         }
+        $cb->();
     };
 
     $cw->show_message("Saving... please wait ...");
@@ -469,11 +471,10 @@ sub save {
 
 }
 
-sub save_if_yes {
+sub quit {
     my $cw = shift;
     my $text = shift || "Save data ?";
 
-    my $ok_to_quit = 1;
     if ( $cw->{root}->instance->needs_save ) {
         my $answer = $cw->Dialog(
             -title          => "quit",
@@ -481,22 +482,23 @@ sub save_if_yes {
             -buttons        => [ qw/yes no cancel/, 'show changes' ],
             -default_button => 'yes',
         )->Show;
-        if ( $answer eq 'yes' ) { $cw->save; $ok_to_quit = 1; }
-        elsif ( $answer eq 'no' )     { $ok_to_quit = 1; }
-        elsif ( $answer eq 'cancel' ) { $ok_to_quit = 0; }
+
+        if ( $answer eq 'yes' ) {
+            $cw->save( sub {$cw->self_destroy;});
+        }
+        elsif ( $answer eq 'no' ) {
+            $cw->self_destroy;
+        }
         elsif ( $answer =~ /show/ ) {
-            $cw->show_changes( sub { $cw->save_if_yes } );
-            $ok_to_quit = 0;
+            $cw->show_changes( sub { $cw->quit } );
         }
     }
 
-    return $ok_to_quit;
 }
 
-sub quit {
-    my $cw = shift;
 
-    $cw->save_if_yes || return;
+sub self_destroy {
+    my $cw = shift;
 
     if ( defined $cw->{quit} and $cw->{quit} eq 'soft' ) {
         $cw->destroy;
